#include <stdint.h>
#include <stdio.h>
#include <TSC_ADC.h>

extern volatile uint32_t *TSC_ADC;
#define SAMPLES 20480
int main(void){
    uint32_t dat[2][SAMPLES]={};
    if(bbb_adc_init()){
        fprintf(stderr, "Can not initialize adc lib\n");
        return(1);
    }

    TSC_ADC[CTRL] = F_StepConfig_WP_OFF;    // stop ADC, disable Step config Write protection
    TSC_ADC[STEPCONFIG1] = F_FIFO_select0|F_SEL_INP_Ch1|F_SEL_INM_ADCREFM|F_Ave4|F_Mode_SW_Con;    // ch1 -> FIFO0, x8 Avg
    TSC_ADC[STEPCONFIG2] = F_FIFO_select1|F_SEL_INP_Ch8|F_SEL_INM_ADCREFM|F_Ave4|F_Mode_SW_Con;    // ch8 -> FIFO1, x8 Avg
//    TSC_ADC[STEPCONFIG1] = F_FIFO_select0|F_SEL_INP_VREFN|F_SEL_INM_ADCREFM|F_Mode_SW_Con;    // ch1 -> FIFO0
//    TSC_ADC[STEPCONFIG2] = F_FIFO_select1|F_SEL_INP_Ch8|F_SEL_INM_ADCREFM|F_Mode_SW_Con;    // ch8 -> FIFO1
    TSC_ADC[IRQSTATUS] = 0xffff;            // clear status flag
    TSC_ADC[IRQENABLE_CLR] = 0xffff;        // disable IRQ
    TSC_ADC[CTRL] = F_Step_ID_tag|F_Enable; // enable ADC

    // make sure to clear IRQSTATUS
    if(TSC_ADC[IRQSTATUS_RAW]){
        fprintf(stderr, "Can not clear IRQSTATUS: set=0x%x cur=0x%x raw=0x%x\n",
            TSC_ADC[IRQENABLE_SET], TSC_ADC[IRQSTATUS], TSC_ADC[IRQSTATUS_RAW]);
        return(1);
    }

    while(TSC_ADC[FIFO0COUNT]){dat[0][0]=TSC_ADC[FIFO0DATA];}   // cleanup FIFO buffer
    TSC_ADC[STEPENABLE] = F_STEP01|F_STEP02;                    // start conversion
    for(int id=0; id < SAMPLES;){
        int c=TSC_ADC[FIFO1COUNT];                              // FIFO1COUNT always larger than FIFO0COUNT
        for(int i=0; i<c; i++){
            dat[0][id] = TSC_ADC[FIFO0DATA];
            dat[1][id] = TSC_ADC[FIFO1DATA];
            id++;
        }
    }
    TSC_ADC[STEPENABLE] = 0x0;                // stop conversion

    for(int i=0; i<SAMPLES; i++){
        printf("%03d, ID%d=%.2f[mV] ID%d=%.2f[mV]\n", i,
            dat[0][i]>>16, 1800*(float)(dat[0][i]&0xfff)/4096,
            dat[1][i]>>16, 1800*(float)(dat[1][i]&0xfff)/4096);
    }

    // error check
    if(TSC_ADC[IRQSTATUS_RAW] & (F_FIFO0_Overrun|F_FIFO1_Overrun|F_FIFO0_Underflow|F_FIFO1_Underflow)){
        fprintf(stderr, "Err: set=0x%x cur=0x%x raw=0x%x\n",
            TSC_ADC[IRQENABLE_SET], TSC_ADC[IRQSTATUS], TSC_ADC[IRQSTATUS_RAW]);
    }

    bbb_adc_free();
    return 0;
}
