#include <stdint.h>
#include <stdio.h>
#include <TSC_ADC.h>

extern volatile uint32_t *TSC_ADC;

int main(void){
    uint32_t dat[2]={};
    if(bbb_adc_init()){
        fprintf(stderr, "Can not initialize adc lib\n");
        return(1);
    }

    TSC_ADC[CTRL] = F_StepConfig_WP_OFF;    // stop ADC, disable Step config Write protection
    TSC_ADC[STEPCONFIG1] = F_FIFO_select0|F_SEL_INP_Ch1|F_SEL_INM_ADCREFM|F_Ave8|F_Mode_SW_One;    // ch1 -> FIFO0, x8 Avg
    TSC_ADC[STEPCONFIG2] = F_FIFO_select0|F_SEL_INP_Ch8|F_SEL_INM_ADCREFM|F_Ave8|F_Mode_SW_One;    // ch8 -> FIFO0, x8 Avg
    TSC_ADC[IRQSTATUS] = 0xffff;        // clear status flag
    TSC_ADC[IRQENABLE_CLR] = 0xffff;    // disable IRQ
    TSC_ADC[CTRL] = F_Step_ID_tag|F_Enable; // enable ADC

    // make sure to clear IRQSTATUS
    if(TSC_ADC[IRQSTATUS_RAW]){
        fprintf(stderr, "Can not clear IRQSTATUS: set=0x%x cur=0x%x raw=0x%x\n",
            TSC_ADC[IRQENABLE_SET], TSC_ADC[IRQSTATUS], TSC_ADC[IRQSTATUS_RAW]);
        return(1);
    }

    while(TSC_ADC[FIFO0COUNT]){dat[0]=TSC_ADC[FIFO0DATA];}  // cleanup FIFO buffer
    TSC_ADC[STEPENABLE] = F_STEP01|F_STEP02;                // start conversion

    while(TSC_ADC[FIFO0COUNT] < 2); // wait sampling
    dat[0] = TSC_ADC[FIFO0DATA];
    dat[1] = TSC_ADC[FIFO0DATA];
    printf("ID%d=%.2f[mV] ID%d=%.2f[mV]\n",
        dat[0]>>16, 1800*(float)(dat[0]&0xfff)/4096,
        dat[1]>>16, 1800*(float)(dat[1]&0xfff)/4096);
    // error check
    if(TSC_ADC[IRQSTATUS_RAW] & (F_FIFO0_Overrun|F_FIFO1_Overrun|F_FIFO0_Underflow|F_FIFO1_Underflow)){
        fprintf(stderr, "Err: set=0x%x cur=0x%x raw=0x%x\n",
            TSC_ADC[IRQENABLE_SET], TSC_ADC[IRQSTATUS], TSC_ADC[IRQSTATUS_RAW]);
    }

    bbb_adc_free();
    return 0;
}
